#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from manukaclient import exceptions


def format_parameters(params, parse_semicolon=True):
    '''Reformat parameters into dict of format expected by the API.'''

    if not params:
        return {}

    if parse_semicolon:
        # expect multiple invocations of --parameters but fall back
        # to ; delimited if only one --parameters is specified
        if len(params) == 1:
            params = params[0].split(';')

    parameters = {}
    for p in params:
        try:
            (n, v) = p.split(('='), 1)
        except ValueError:
            msg = 'Malformed parameter(%s). Use the key=value format.' % p
            raise exceptions.CommandError(msg)

        if n not in parameters:
            parameters[n] = v
        else:
            if not isinstance(parameters[n], list):
                parameters[n] = [parameters[n]]
            parameters[n].append(v)

    return parameters



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from keystoneauth1 import adapter
from oslo_utils import importutils

from manukaclient import exceptions


def Client(version, *args, **kwargs):
    module = 'manukaclient.v%s.client' % version
    module = importutils.import_module(module)
    client_class = getattr(module, 'Client')
    return client_class(*args, **kwargs)


class SessionClient(adapter.Adapter):

    client_name = 'python-manukaclient'
    client_version = '0.1.0'

    def request(self, url, method, **kwargs):
        project_id = self.get_project_id()
        kwargs.setdefault('headers', kwargs.get('headers', {}))
        kwargs['headers']['X-PROJECT-ID'] = project_id
        # NOTE(sorrison): The standard call raises errors from
        # keystoneauth, where we need to raise the manukaclient errors.
        raise_exc = kwargs.pop('raise_exc', True)
        resp = super(SessionClient, self).request(url,
                                                  method,
                                                  raise_exc=False,
                                                  **kwargs)

        if raise_exc and resp.status_code >= 400:
            raise exceptions.from_response(resp, url, method)
        # NOTE(sorrison): Deletes don't return json body
        if resp.status_code == 204:
            return resp, '{}'
        return resp, resp.json()



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import abc
import copy

from requests import Response
import six

from manukaclient import exceptions


def getid(obj):
    """Get obj's id or object itself if no id
    Abstracts the common pattern of allowing both an object or
    an object's ID as a parameter when dealing with relationships.
    """
    try:
        return obj.id
    except AttributeError:
        return obj


class Manager(object):
    """Interacts with type of API
    Managers interact with a particular type of API (instances, types, etc.)
    and provide CRUD operations for them.
    """
    resource_class = None

    def __init__(self, api):
        self.api = api

    def _list(self, url, response_key='results', obj_class=None,
              items=None, headers=None, params=None, limit=None):

        if items is None:
            items = []
        if headers is None:
            headers = {}
        resp, body = self.api.get(url, headers=headers, params=params)

        if obj_class is None:
            obj_class = self.resource_class

        if response_key and response_key in body:
            data = body[response_key]
        else:
            data = body

        if all([isinstance(res, six.string_types) for res in data]):
            new_items = data
        else:
            new_items = [obj_class(self, res, loaded=True)
                         for res in data if res]

        items = items + new_items
        if 'next' in body and body['next']:
            items = self._list(body['next'], response_key, obj_class, items,
                               headers, None, limit)

        return ListWithMeta(items, resp)

    def _delete(self, url, headers=None):
        if headers is None:
            headers = {}
        resp, body = self.api.delete(url, headers=headers)

        return self.convert_into_with_meta(body, resp)

    def _update(self, url, data, response_key=None, return_raw=False,
                headers=None):
        if headers is None:
            headers = {}
        resp, body = self.api.patch(url, data=data, headers=headers)
        if return_raw:
            if response_key:
                body = body[response_key]
            return self.convert_into_with_meta(body, resp)
        # PATCH requests may not return a body
        if body:
            if response_key:
                return self.resource_class(self, body[response_key], resp=resp)
            return self.resource_class(self, body, resp=resp)
        else:
            return StrWithMeta(body, resp)

    def _update_all(self, url, data, response_key=None, return_raw=False,
                    headers=None):
        if headers is None:
            headers = {}
        resp, body = self.api.put(url, data=data, headers=headers)
        if return_raw:
            if response_key:
                body = body[response_key]
            return self.convert_into_with_meta(body, resp)
        # PUT requests may not return a body
        if body:
            if response_key:
                return self.resource_class(self, body[response_key], resp=resp)
            return self.resource_class(self, body, resp=resp)
        else:
            return StrWithMeta(body, resp)

    def _create(self, url, data=None, response_key=None, return_raw=False,
                headers=None):
        if headers is None:
            headers = {}
        if data:
            resp, body = self.api.post(url, data=data, headers=headers)
        else:
            resp, body = self.api.post(url, headers=headers)
        if return_raw:
            if response_key:
                body = body[response_key]
            return self.convert_into_with_meta(body, resp)

        if response_key:
            return self.resource_class(self, body[response_key], resp=resp)
        return self.resource_class(self, body, resp=resp)

    def _get(self, url, response_key=None, return_raw=False, headers=None):
        if headers is None:
            headers = {}
        resp, body = self.api.get(url, headers=headers)
        if return_raw:
            if response_key:
                body = body[response_key]
            return self.convert_into_with_meta(body, resp)

        if response_key:
            return self.resource_class(self, body[response_key], loaded=True,
                                       resp=resp)
        return self.resource_class(self, body, loaded=True, resp=resp)

    def convert_into_with_meta(self, item, resp):
        if isinstance(item, six.string_types):
            if six.PY2 and isinstance(item, six.text_type):
                return UnicodeWithMeta(item, resp)
            else:
                return StrWithMeta(item, resp)
        elif isinstance(item, six.binary_type):
            return BytesWithMeta(item, resp)
        elif isinstance(item, list):
            return ListWithMeta(item, resp)
        elif isinstance(item, tuple):
            return TupleWithMeta(item, resp)
        elif item is None:
            return TupleWithMeta((), resp)
        else:
            return DictWithMeta(item, resp)


@six.add_metaclass(abc.ABCMeta)
class ManagerWithFind(Manager):
    """Manager with additional `find()`/`findall()` methods."""

    @abc.abstractmethod
    def list(self):
        pass

    def find(self, **kwargs):
        """Find a single item with attributes matching ``**kwargs``.
        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        matches = self.findall(**kwargs)
        num = len(matches)

        if num == 0:
            msg = "No %s matching %s." % (self.resource_class.__name__, kwargs)
            raise exceptions.NotFound(msg)
        elif num > 1:
            raise exceptions.NoUniqueMatch
        else:
            return self.get(matches[0].id)

    def findall(self, **kwargs):
        """Find all items with attributes matching ``**kwargs``.
        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        found = []
        searches = kwargs.items()

        for obj in self.list():
            try:
                if all(getattr(obj, attr) == value
                       for (attr, value) in searches):
                    found.append(obj)
            except AttributeError:
                continue

        return found


class BasicManager(ManagerWithFind):

    def list(self, **kwargs):
        return self._list('/%s/' % self.base_url, params=kwargs)

    def get(self, resource_id):
        return self._get('/%s/%s/' % (self.base_url, resource_id))


class RequestIdMixin(object):
    """Wrapper class to expose x-openstack-request-id to the caller."""
    def request_ids_setup(self):
        self.x_openstack_request_ids = []

    @property
    def request_ids(self):
        return self.x_openstack_request_ids

    def append_request_ids(self, resp):
        """Add request_ids as an attribute to the object
        :param resp: Response object or list of Response objects
        """
        if isinstance(resp, list):
            # Add list of request_ids if response is of type list.
            for resp_obj in resp:
                self._append_request_id(resp_obj)
        elif resp is not None:
            # Add request_ids if response contains single object.
            self._append_request_id(resp)

    def _append_request_id(self, resp):
        if isinstance(resp, Response):
            # Extract 'X-Openstack-Request-Id' from headers if
            # response is a Response object.
            request_id = (resp.headers.get('Openstack-Request-Id')
                          or resp.headers.get('x-openstack-request-id')
                          or resp.headers.get('x-compute-request-id'))
        else:
            # If resp is of type string or None.
            request_id = resp
        if request_id not in self.x_openstack_request_ids:
            self.x_openstack_request_ids.append(request_id)


class Resource(RequestIdMixin):
    """Represents an instance of an object
    A resource represents a particular instance of an object (instance, type,
    etc). This is pretty much just a bag for attributes.
    :param manager: BaseManager object
    :param info: dictionary representing resource attributes
    :param loaded: prevent lazy-loading if set to True
    :param resp: Response or list of Response objects
    """

    def __init__(self, manager, info, loaded=False, resp=None):
        self.manager = manager
        self._info = info
        self._add_details(info)
        self._loaded = loaded
        self.request_ids_setup()
        self.append_request_ids(resp)

    def _add_details(self, info):
        for (k, v) in six.iteritems(info):
            try:
                setattr(self, k, v)
                self._info[k] = v
            except AttributeError:
                # In this case we already defined the attribute on the class
                pass

    def __setstate__(self, d):
        for k, v in d.items():
            setattr(self, k, v)

    def __getattr__(self, k):
        if k not in self.__dict__:
            # NOTE(RuiChen): disallow lazy-loading if already loaded once
            if not self.is_loaded():
                self.get()
                return self.__getattr__(k)
            raise AttributeError(k)
        else:
            return self.__dict__[k]

    def __repr__(self):
        reprkeys = sorted(k for k in self.__dict__.keys() if k[0] != '_'
                          and k not in ('manager', 'x_openstack_request_ids'))
        info = ", ".join("%s=%s" % (k, getattr(self, k)) for k in reprkeys)
        return "<%s %s>" % (self.__class__.__name__, info)

    def get(self):
        # set_loaded() first ... so if we have to bail, we know we tried.
        self.set_loaded(True)
        if not hasattr(self.manager, 'get'):
            return

        new = self.manager.get(self.id)
        if new:
            self._add_details(new._info)
            # The 'request_ids' attribute has been added,
            # so store the request id to it instead of _info
            self.append_request_ids(new.request_ids)

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        return self._info == other._info

    def __ne__(self, other):
        return not self.__eq__(other)

    def is_loaded(self):
        return self._loaded

    def set_loaded(self, val):
        self._loaded = val

    def to_dict(self):
        return copy.deepcopy(self._info)


class ListWithMeta(list, RequestIdMixin):
    def __init__(self, values, resp):
        super(ListWithMeta, self).__init__(values)
        self.request_ids_setup()
        self.append_request_ids(resp)


class DictWithMeta(dict, RequestIdMixin):
    def __init__(self, values, resp):
        super(DictWithMeta, self).__init__(values)
        self.request_ids_setup()
        self.append_request_ids(resp)


class TupleWithMeta(tuple, RequestIdMixin):
    def __new__(cls, values, resp):
        return super(TupleWithMeta, cls).__new__(cls, values)

    def __init__(self, values, resp):
        self.request_ids_setup()
        self.append_request_ids(resp)


class StrWithMeta(str, RequestIdMixin):
    def __new__(cls, value, resp):
        return super(StrWithMeta, cls).__new__(cls, value)

    def __init__(self, values, resp):
        self.request_ids_setup()
        self.append_request_ids(resp)


class BytesWithMeta(six.binary_type, RequestIdMixin):
    def __new__(cls, value, resp):
        return super(BytesWithMeta, cls).__new__(cls, value)

    def __init__(self, values, resp):
        self.request_ids_setup()
        self.append_request_ids(resp)


if six.PY2:
    class UnicodeWithMeta(six.text_type, RequestIdMixin):
        def __new__(cls, value, resp):
            return super(UnicodeWithMeta, cls).__new__(cls, value)

        def __init__(self, values, resp):
            self.request_ids_setup()
            self.append_request_ids(resp)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from osc_lib import utils


DEFAULT_API_VERSION = '1'

# Required by the OSC plugin interface
API_NAME = 'account'
API_VERSION_OPTION = 'os_account_api_version'
API_VERSIONS = {
    '1': 'manukaclient.v1.client.Client',
}


# Required by the OSC plugin interface
def make_client(instance):
    """Returns a client to the ClientManager

    Called to instantiate the requested client version.  instance has
    any available auth info that may be required to prepare the client.

    :param ClientManager instance: The ClientManager that owns the new client
    """
    plugin_client = utils.get_client_class(
        API_NAME,
        instance._api_version[API_NAME],
        API_VERSIONS)

    endpoint = instance._cli_options.account_endpoint
    client = plugin_client(session=instance.session,
                           endpoint_override=endpoint)
    return client


# Required by the OSC plugin interface
def build_option_parser(parser):
    """Hook to add global options

    Called from openstackclient.shell.OpenStackShell.__init__()
    after the builtin parser has been initialized.  This is
    where a plugin can add global options such as an API version setting.

    :param argparse.ArgumentParser parser: The parser object that has been
        initialized by OpenStackShell.
    """
    parser.add_argument(
        '--os-account-api-version',
        metavar='<account-api-version>',
        help='Nectar Account API version, default='
             + DEFAULT_API_VERSION
             + ' (Env: OS_ACCOUNT_API_VERSION)')
    parser.add_argument(
        '--os-account-endpoint',
        metavar='<account-endpoint>',
        help='Nectar Account API endpoint')
    return parser



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import json

from manukaclient import base


class User(base.Resource):
    pass


class UserManager(base.BasicManager):

    base_url = 'v1/users'
    resource_class = User

    def update(self, user_id, **kwargs):
        data = json.dumps(kwargs)
        return self._update('/%s/%s/' % (self.base_url, user_id), data=data,
                            headers={"content-type": "application/json"})

    def get_by_os(self, user_id):
        return self._get('v1/users-os/%s/' % user_id)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import logging

from osc_lib.command import command
from osc_lib import utils as osc_utils

from manukaclient import exceptions
from manukaclient.osc import utils


class ListUsers(command.Lister):
    """List users."""

    log = logging.getLogger(__name__ + '.ListUsers')

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        client = self.app.client_manager.account
        users = client.users.list()
        columns = ['id', 'user_id', 'email']
        return (
            columns,
            (osc_utils.get_item_properties(q, columns) for q in users)
        )


class ShowUser(command.ShowOne):
    """Show user details."""

    log = logging.getLogger(__name__ + '.ShowUser')

    def get_parser(self, prog_name):
        parser = super(ShowUser, self).get_parser(prog_name)
        parser.add_argument(
            'id',
            metavar='<id>',
            help=('ID of user')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        client = self.app.client_manager.account
        try:
            user = client.users.get(parsed_args.id)
        except exceptions.NotFound as ex:
            raise exceptions.CommandError(str(ex))

        return self.dict2columns(user.to_dict())


class UpdateUser(command.ShowOne):
    """Update an user."""

    log = logging.getLogger(__name__ + '.UpdateUser')

    def get_parser(self, prog_name):
        parser = super(UpdateUser, self).get_parser(prog_name)
        parser.add_argument(
            'id',
            metavar='<id>',
            help=('ID of user')
        )
        parser.add_argument(
            '--property',
            metavar='<key=value>',
            action='append',
            help=('Properties to set on the user. This can be '
                  'specified multiple times'))
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        client = self.app.client_manager.account
        try:
            user = client.users.get(parsed_args.id)
        except exceptions.NotFound as ex:
            raise exceptions.CommandError(str(ex))
        fields = utils.format_parameters(parsed_args.property)

        user = client.users.update(parsed_args.id, **fields)
        return self.dict2columns(user.to_dict())



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

import mock
from six.moves.urllib import parse

from manukaclient import client as base_client
from manukaclient.tests.unit import fakes
from manukaclient.tests.unit import utils
from manukaclient.v1 import client
from manukaclient.v1 import users


# regex to compare callback to result of get_endpoint()
# checks version number (vX or vX.X where X is a number)
# and also checks if the id is on the end
ENDPOINT_RE = re.compile(
    r"^get_http:__manuka_api:8774_v\d(_\d)?_\w{32}$")

# accepts formats like v2 or v2.1
ENDPOINT_TYPE_RE = re.compile(r"^v\d(\.\d)?$")

# accepts formats like v2 or v2_1
CALLBACK_RE = re.compile(r"^get_http:__manuka_api:8774_v\d(_\d)?$")

generic_user = {
    "first_name": "uEelNrtNg3SPzh50nol5",
    "affiliation": "staff",
    "id": 123,
    "home_organization": "saDjCIriNGoJKqUI4piX",
    "mobile_number": "6Q1llL0jyOhNdyBAX0XO",
    "terms_accepted_at": "2015-10-21T15:18:40",
    "surname": "OIEBQswE3tNh89N5OTsW",
    "user_id": "d1fa8867e42444cf8724e65fef1da549",
    "phone_number": "33443322",
    "displayname": "lyWtLuxXWxku24cbhgjT",
    "registered_at": "2015-10-21T15:18:40",
    "email": "fmklmf4ikmlf34mnm",
    "ignore_username_not_email": False,
    "orcid": "sammmee",
    "state": "created",
    "last_login": "2020-04-23T10:23:20",
    "terms_version": "v1"
}


class FakeClient(fakes.FakeClient, client.Client):

    def __init__(self, *args, **kwargs):
        client.Client.__init__(self, session=mock.Mock())
        self.http_client = FakeSessionClient(**kwargs)
        self.users = users.UserManager(self.http_client)


class FakeSessionClient(base_client.SessionClient):

    def __init__(self, *args, **kwargs):

        self.callstack = []
        self.visited = []
        self.auth = mock.Mock()
        self.session = mock.Mock()
        self.service_type = 'service_type'
        self.service_name = None
        self.endpoint_override = None
        self.interface = None
        self.region_name = None
        self.version = None
        self.auth.get_auth_ref.return_value.project_id = 'tenant_id'
        # determines which endpoint to return in get_endpoint()
        # NOTE(augustina): this is a hacky workaround, ultimately
        # we need to fix our whole mocking architecture (fixtures?)
        if 'endpoint_type' in kwargs:
            self.endpoint_type = kwargs['endpoint_type']
        else:
            self.endpoint_type = 'endpoint_type'
        self.logger = mock.MagicMock()

    def request(self, url, method, **kwargs):
        return self._cs_request(url, method, **kwargs)

    def _cs_request(self, url, method, **kwargs):
        # Check that certain things are called correctly
        if method in ['GET', 'DELETE']:
            assert 'data' not in kwargs
        elif method == 'PUT':
            assert 'data' in kwargs

        if url is not None:
            # Call the method
            args = parse.parse_qsl(parse.urlparse(url)[4])
            kwargs.update(args)
            munged_url = url.rsplit('?', 1)[0]
            munged_url = munged_url.strip('/').replace('/', '_')
            munged_url = munged_url.replace('.', '_')
            munged_url = munged_url.replace('-', '_')
            munged_url = munged_url.replace(' ', '_')
            munged_url = munged_url.replace('!', '_')
            munged_url = munged_url.replace('@', '_')
            munged_url = munged_url.replace('%20', '_')
            munged_url = munged_url.replace('%3A', '_')
            callback = "%s_%s" % (method.lower(), munged_url)

        if not hasattr(self, callback):
            raise AssertionError('Called unknown API method: %s %s, '
                                 'expected fakes method name: %s' %
                                 (method, url, callback))

        # Note the call
        self.visited.append(callback)
        self.callstack.append((method, url, kwargs.get('data'),
                               kwargs.get('params')))

        status, headers, data = getattr(self, callback)(**kwargs)

        r = utils.TestResponse({
            "status_code": status,
            "text": data,
            "headers": headers,
        })
        return r, data

    def get_v1_users(self, **kw):
        users = [
            {
                "first_name": "uEelNrtNg3SPzh50nol5",
                "affiliation": "staff",
                "id": 123,
                "home_organization": "saDjCIriNGoJKqUI4piX",
                "mobile_number": "6Q1llL0jyOhNdyBAX0XO",
                "terms_accepted_at": "2015-10-21T15:18:40",
                "surname": "OIEBQswE3tNh89N5OTsW",
                "user_id": "d1fa8867e42444cf8724e65fef1da549",
                "phone_number": "33443322",
                "displayname": "lyWtLuxXWxku24cbhgjT",
                "registered_at": "2015-10-21T15:18:40",
                "email": "fmklmf4ikmlf34mnm",
                "ignore_username_not_email": False,
                "orcid": "sammmee",
                "state": "created",
                "last_login": "2020-04-23T10:23:20",
                "terms_version": "v1"
            },
            {
                "first_name": "uEelNrtNg3SPzh50nol5",
                "affiliation": "staff",
                "id": 124,
                "home_organization": "saDjCIriNGoJKqUI4piX",
                "mobile_number": "6Q1llL0jyOhNdyBAX0XO",
                "terms_accepted_at": "2015-10-21T15:18:40",
                "surname": "OIEBQswE3tNh89N5OTsW",
                "user_id": "d1fa8867e42444cf8724e65fef1da549",
                "phone_number": "33443322",
                "displayname": "lyWtLuxXWxku24cbhgjT",
                "registered_at": "2015-10-21T15:18:40",
                "email": "fmklmf4ikmlf34mnm",
                "ignore_username_not_email": False,
                "orcid": "sammmee",
                "state": "created",
                "last_login": "2020-04-23T10:23:20",
                "terms_version": "v1"
            },
            {
                "first_name": "uEelNrtNg3SPzh50nol5",
                "affiliation": "staff",
                "id": 125,
                "home_organization": "saDjCIriNGoJKqUI4piX",
                "mobile_number": "6Q1llL0jyOhNdyBAX0XO",
                "terms_accepted_at": "2015-10-21T15:18:40",
                "surname": "OIEBQswE3tNh89N5OTsW",
                "user_id": "d1fa8867e42444cf8724e65fef1da549",
                "phone_number": "33443322",
                "displayname": "lyWtLuxXWxku24cbhgjT",
                "registered_at": "2015-10-21T15:18:40",
                "email": "fmklmf4ikmlf34mnm",
                "ignore_username_not_email": False,
                "orcid": "sammmee",
                "state": "created",
                "last_login": "2020-04-23T10:23:20",
                "terms_version": "v1"
            }
        ]
        return (200, {}, users)

    def get_v1_users_123(self, **kw):
        return (200, {}, generic_user)

    def patch_v1_users_123(self, data, **kw):
        return (202, {'orcid': 'new-orcid'},
            {
                "first_name": "uEelNrtNg3SPzh50nol5",
                "affiliation": "staff",
                "id": 123,
                "home_organization": "saDjCIriNGoJKqUI4piX",
                "mobile_number": "6Q1llL0jyOhNdyBAX0XO",
                "terms_accepted_at": "2015-10-21T15:18:40",
                "surname": "OIEBQswE3tNh89N5OTsW",
                "user_id": "d1fa8867e42444cf8724e65fef1da549",
                "phone_number": "33443322",
                "displayname": "lyWtLuxXWxku24cbhgjT",
                "registered_at": "2015-10-21T15:18:40",
                "email": "fmklmf4ikmlf34mnm",
                "ignore_username_not_email": False,
                "orcid": "new-orcid",
                "state": "created",
                "last_login": "2020-04-23T10:23:20",
                "terms_version": "v1"
            })



#!/usr/bin/env python

import setuptools

from pbr.packaging import parse_requirements

entry_points = {
    'openstack.cli.extension':
    ['account = manukaclient.osc.plugin'],
    'openstack.account.v1':
    [
        'account user list = manukaclient.osc.v1.users:ListUsers',
        'account user show = manukaclient.osc.v1.users:ShowUser',
        'account user set = manukaclient.osc.v1.users:UpdateUser',
    ]
}


setuptools.setup(
    name='manukaclient',
    version='0.1.0',
    description=('Client for the Nectar Account system'),
    author='Sam Morrison',
    author_email='sorrison@gmail.com',
    url='https://github.com/NeCTAR-RC/python-manukaclient',
    packages=[
        'manukaclient',
    ],
    include_package_data=True,
    setup_requires=['pbr>=3.0.0'],
    install_requires=parse_requirements(),
    license="Apache",
    zip_safe=False,
    classifiers=(
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ),
    entry_points=entry_points,
)
